library flutter_payphone;

import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:progress_dialog/progress_dialog.dart';

class Payphone {
  String tokenApiKey;

  Payphone(this.tokenApiKey);

  Future<Map<String, dynamic>> enviarVentaResultadoWidget(
      BuildContext context,
      phoneNumber,
      amount,
      amountWithTax,
      tax,
      clientTransactionId,
      referencia) async {
    ProgressDialog pr = ProgressDialog(
      context,
      isDismissible: false,
    );
    pr.style(
        message: 'Solicitando pago a Payphone...',
        borderRadius: 10.0,
        backgroundColor: Colors.white,
        progressWidget: CircularProgressIndicator(),
        elevation: 10.0,
        insetAnimCurve: Curves.easeInOut,
        progress: 0.0,
        maxProgress: 100.0,
        progressTextStyle: TextStyle(
            color: Colors.black, fontSize: 13.0, fontWeight: FontWeight.w400),
        messageTextStyle: TextStyle(
            color: Colors.black, fontSize: 19.0, fontWeight: FontWeight.w600));
    await pr.show();
    await Future.delayed(Duration(seconds: 2));
    pr.update(
        message: "Esperando que usuario acepte el pago en la APP Payphone...");
    Map<String, dynamic> resultado = await enviarVentaResultado(phoneNumber,
        amount, amountWithTax, tax, clientTransactionId, referencia);
    await pr.hide();
    return resultado;
  }

  Future<Map<String, dynamic>> enviarVentaResultado(phoneNumber, amount,
      amountWithTax, tax, clientTransactionId, referencia) async {
    Map<String, dynamic> respuestaSolicitudVenta = await _saleRequest(
        phoneNumber,
        amount,
        amountWithTax,
        tax,
        clientTransactionId,
        referencia);
    if (respuestaSolicitudVenta['transactionId'] != null) {
      Map<String, dynamic> status = await consultarVenta(clientTransactionId);
      //print("status fuera while: ${status.toString()}");
      int statusCode = status['statusCode'];
      while (statusCode == '1' /* PENDIENTE */) {
        await Future.delayed(Duration(seconds: 5));
        status = await consultarVenta(clientTransactionId);
        //print("status dentro while: ${status.toString()}");
        statusCode = status['statusCode'];
      }
      return status;
    } else {
      return respuestaSolicitudVenta;
    }
  }

  Future<Map<String, dynamic>> _saleRequest(phoneNumber, amount, amountWithTax,
      tax, clientTransactionId, referencia) async {
    var url = 'https://pay.payphonetodoesposible.com/api/Sale';
    var response = await http.post(
      url,
      headers: {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': 'Bearer ' + tokenApiKey,
      },
      body: jsonEncode(<String, dynamic>{
        if (phoneNumber != null) 'phoneNumber': phoneNumber,
        'countryCode': "593",
        /* Ecuador */
        if (amount != null)
          'amount': amount,
        if (amountWithTax != null) 'amountWithTax': amountWithTax,
        if (tax != null) 'tax': tax,
        if (referencia != null) 'referencia': referencia,
        if (clientTransactionId != null)
          'clientTransactionId': clientTransactionId,
        'optionalParameter1':
            "Transacción procesada con el plugin para flutter: flutter_payphone"
      }),
    );

    return json.decode(response.body);
  }

  Future<Map<String, dynamic>> consultarVenta(clientTransactionId) async {
    List<dynamic> lista;
    Map<String, dynamic> resultado;
    var url = 'https://pay.payphonetodoesposible.com/api/Sale/client/' +
        clientTransactionId;
    var response = await http.get(
      url,
      headers: {
        'Accept': 'application/json',
        'Authorization': 'Bearer ' + tokenApiKey,
      },
    );
    lista = json.decode(response.body);
    resultado = lista.first;
    return resultado;
  }

  Future<Map<String, dynamic>> enviarAnulacionResultado(
      clientTransactionId) async {
    Map<String, dynamic> respuestaSolicitudAnulacion =
        await _annulRequest(clientTransactionId);
    if (respuestaSolicitudAnulacion['id'] != null) {
      Map<String, dynamic> status =
          await consultarAnulacion(clientTransactionId);
      //print("status fuera while: ${status.toString()}");
      int statusCode = status['statusCode'];
      while (statusCode == '1' /* PENDIENTE */) {
        await Future.delayed(Duration(seconds: 5));
        status = await consultarAnulacion(clientTransactionId);
        //print("status dentro while: ${status.toString()}");
        statusCode = status['statusCode'];
      }
      return status;
    } else {
      return respuestaSolicitudAnulacion;
    }
  }

  Future<Map<String, dynamic>> _annulRequest(clientTransactionId) async {
    var url = 'https://pay.payphonetodoesposible.com/api/Annul/client';
    var response = await http.post(
      url,
      headers: {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': 'Bearer ' + tokenApiKey,
      },
      body: jsonEncode(<String, dynamic>{
        'clientId': clientTransactionId,
      }),
    );

    return json.decode(response.body);
  }

  Future<Map<String, dynamic>> consultarAnulacion(clientTransactionId) async {
    Map<String, dynamic> resultado;
    var url = 'https://pay.payphonetodoesposible.com/api/Annul/client/' +
        clientTransactionId;
    var response = await http.get(
      url,
      headers: {
        'Accept': 'application/json',
        'Authorization': 'Bearer ' + tokenApiKey,
      },
    );
    return json.decode(response.body);
  }
}
